! Copyright (c) 1999-2002 by Sand & Associates

  MEMBER()
  INCLUDE('SaTzi.inc'),ONCE

  MAP
   MODULE('Kernal32')
     GetTimeZoneInformation(*TIME_ZONE_INFORMATION),LONG,PASCAL,RAW

     !-----------------------------------------------------------------------
     ! The WideCharToMultiByte function maps a wide-character string to a
     ! new character string. The new character string is not necessarily
     ! from a multibyte character set, it can be from the ANSI character set.
     !-----------------------------------------------------------------------
     WideCharToMultiByte(          |
          UNSIGNED CodePage,       | ! code page to map the characters to
          UNSIGNED dwFlags,        | ! performance and mapping flags
          LONG lpWideCharStr,      | ! address of wide-character string
          LONG cchWideChar,        | ! number of characters in string (-1 means Null terminated)
          *CSTRING lpMultiByteStr, | ! address of buffer for new string, this will be the ANSI string
          LONG cbMultiByte,        | ! size of buffer for new string
          LONG lpDefaultChar=0,    | ! address of default for unmappable characters
          LONG lpUsedDefaultChar=0 | ! address of flag set when default characters used
        ),LONG,PASCAL,RAW,PROC       ! Returns number of bytes written (including null) or 0 on failure

     GetSystemTime(*SYSTEMTIME),PASCAL,RAW

     GetLastError(),LONG,PASCAL,PROC

   END
  END

!-----------------------------------------------------------------
! GetTimeZoneInfo
!
! Get the time zone information from Windows and place the results
! into the class properties
!-----------------------------------------------------------------
SaTziClass.GetTimeZoneInfo PROCEDURE()

  CODE
  SELF.TziReturn = GetTimeZoneInformation(SELF.Tzi)

  IF SELF.TziReturn = TIME_ZONE_ID_INVALID
    SELF.LastErrorCode = GetLastError()
    CLEAR(SELF.Tzi)
    RETURN SELF.LastErrorCode

  ELSE
    ! Convert the Wide Strings to ANSI Strings
    IF SELF.W2A(ADDRESS(SELF.Tzi.StandardName), SELF.StandardZoneName) = 0
      SELF.LastErrorCode = GetLastError()
      RETURN SELF.LastErrorCode
    END
    IF SELF.W2A(ADDRESS(SELF.Tzi.DaylightName), SELF.DaylightZoneName) = 0
      SELF.LastErrorCode = GetLastError()
      RETURN SELF.LastErrorCode
    END

    CASE SELF.TziReturn
    OF TIME_ZONE_ID_STANDARD   !Windows 95 returns this for No Daylight time
      SELF.CurrentZoneName = SELF.StandardZoneName
      SELF.ActiveTimeBias = SELF.Tzi.Bias + SELF.Tzi.StandardBias

    OF TIME_ZONE_ID_DAYLIGHT
      SELF.CurrentZoneName = SELF.DaylightZoneName
      SELF.ActiveTimeBias = SELF.Tzi.Bias + SELF.Tzi.DaylightBias

    OF TIME_ZONE_ID_UNKNOWN    !NT/2000 return this for No Daylight time

      IF SELF.Tzi.Bias  !No Daylight time so use Standard time
        SELF.CurrentZoneName = SELF.StandardZoneName
        SELF.ActiveTimeBias = SELF.Tzi.Bias
      ELSE
        SELF.CurrentZoneName = 'Unknown'
        SELF.ActiveTimeBias  = 0
      END
    END

    !Save the dates and time in the class properties in Clarion friendly format
    SELF.GetStartDate(SELF.StandardStartDateStr, SELF.Tzi.StandardDate)
    SELF.GetStartDate(SELF.DaylightStartDateStr, SELF.Tzi.DaylightDate)

    SELF.StandardStartDate = SELF.GetStartDate(SELF.Tzi.StandardDate)
    SELF.DaylightStartDate = SELF.GetStartDate(SELF.Tzi.DaylightDate)

    SELF.StandardStartTime = SELF.GetStartTime(SELF.Tzi.StandardDate)
    SELF.DaylightStartTime = SELF.GetStartTime(SELF.Tzi.DaylightDate)
  END
  RETURN TZI_SUCCESS



!-----------------------------------------------------------------
! GetStartTime
!
! Get the time zone transition time.  Returns a Clarion Standard
! Time or TZI_NOTRANSITION for no transition time.
!-----------------------------------------------------------------
SaTziClass.GetStartTime PROCEDURE(*SYSTEMTIME st)
StartTime   LONG,AUTO
  CODE                                            
  IF SELF.TziReturn <> TIME_ZONE_ID_UNKNOWN  AND (st.wMonth <> 0 AND st.wDay <> 0)
    StartTime = SELF.SystemTimeToClarionTime(st)
  ELSE
    StartTime = TZI_NOTRANSITION
  END
  RETURN StartTime




!-----------------------------------------------------------------
! SystemTimeToClarionTime
!
! Convert a Windows system time structure to a Clarion standard time
!-----------------------------------------------------------------
SaTziClass.SystemTimeToClarionTime  PROCEDURE(*SYSTEMTIME st)
  CODE
  RETURN (st.wHour*360000 + st.wMinute*6000 + st.wSecond*100 + 1)



!-----------------------------------------------------------------
! LocalTimeToUTC
!
! Convert a the local time in Clarion standard time format to UTC.
! UTC returned as a Clarion standard time
!-----------------------------------------------------------------
SaTziClass.LocalTimeToUTC   PROCEDURE(LONG LocalTime)
  CODE
  RETURN (LocalTime + SELF.ActiveTimeBias*6000)



!-----------------------------------------------------------------
! UTCToLocalTime
!
! Convert a UTC in Clarion standard time format to the Local time.
! Local time returned as a Clarion standard time.
!-----------------------------------------------------------------
SaTziClass.UTCToLocalTime   PROCEDURE(LONG UTC)
  CODE
  RETURN (UTC - SELF.ActiveTimeBias*6000)



!-----------------------------------------------------------------
! GetStartDate
!
! Return a string describing the starting date or '' for no
! transition date.
!-----------------------------------------------------------------
SaTziClass.GetStartDate     PROCEDURE(*CSTRING DateStr, *SYSTEMTIME st)

DayOfWeekStr        STRING('Sunday         ' &|
                           'Monday         ' &|
                           'Tuesday        ' &|
                           'Wednesday      ' &|
                           'Thursday       ' &|
                           'Friday         ' &|
                           'Saturday       ')
DayOfWeek           STRING(15),DIM(7),OVER(DayOfWeekStr)

WeekOfMonthStr      STRING('First     ' &|
                           'Second    ' &|
                           'Third     ' &|
                           'Fourth    ' &|
                           'Last      ')
WeekOfMonth         STRING(10),DIM(5),OVER(WeekOfMonthStr)

MonthOfYearStr      STRING('January        ' &|
                           'February       ' &|
                           'March          ' &|
                           'April          ' &|
                           'May            ' &|
                           'June           ' &|
                           'July           ' &|
                           'August         ' &|
                           'September      ' &|
                           'October        ' &|
                           'November       ' &|
                           'December       ')
MonthOfYear         STRING(15)DIM(12),OVER(MonthOfYearStr)
eINstr      EQUATE('in')

  CODE
  DateStr = ''
  IF SELF.TziReturn = TIME_ZONE_ID_UNKNOWN OR (st.wMonth = 0 OR st.wDay = 0)
    RETURN
  END

  IF SELF.DateFormat = '' THEN SELF.DateFormat = '@D17'.
  IF st.wYear = 0      !Date is in Day-in-Month format
    DateStr = CLIP(WeekOfMonth[st.wDay])&' '&|
              CLIP(DayOfWeek[st.wDayOfWeek+1])&' '&eINstr&' '&|
              CLIP(MonthOfYear[st.wMonth])
  ELSE
    DateStr = FORMAT(DATE(st.wMonth, st.wDay, st.wYear),SELF.DateFormat)
  END

  RETURN



!-----------------------------------------------------------------
! GetStartDate
!
! Return the transition date in a Clarion Standard Date or
! TZI_NOTRANSITION for no transition date.
!-----------------------------------------------------------------
SaTziClass.GetStartDate     PROCEDURE(*SYSTEMTIME st)
StartDate   LONG,AUTO
  CODE
  IF SELF.TziReturn <> TIME_ZONE_ID_UNKNOWN AND (st.wMonth <> 0 AND st.wDay <> 0)
    IF SELF.tzYear = 0 THEN SELF.tzYear = YEAR(TODAY()).
    IF st.wYear = 0      !Date is in Day-in-Month format
      StartDate = SELF.DayInMonthToClarionDate(SELF.tzYear, st)
    ELSE
      StartDate = DATE(st.wMonth, st.wDay, st.wYear)
    END
  ELSE
    StartDate = TZI_NOTRANSITION
  END

  RETURN StartDate



!-----------------------------------------------------------------
! DayInMonthToClarionDate
!
! Transform a tzi day-in-month form date to a Clarion Standard date.
! An example of the day-in-month form is the "Last Sunday in October"
! in this case the sytem time structure contains:
!         st.wMonth = 10       !October
!         st.wDayOfWeek = 0    !Sunday
!         st.wDay = 5          !Last
!-----------------------------------------------------------------
SaTziClass.DayInMonthToClarionDate  PROCEDURE(LONG xYear, *SYSTEMTIME st)
ThisDay         USHORT,AUTO
LastDay         USHORT,AUTO
DayCount        BYTE,AUTO
ThisDate     LONG,AUTO
  CODE
  DayCount = 0
  LastDay = 0
  LOOP ThisDay = 1 TO 31
    ThisDate = DATE(st.wMonth, ThisDay, xYear)
    IF ThisDate % 7 = st.wDayOfWeek   !Correct day of week?
      DayCount += 1
      IF DayCount = st.wDay  !1=First, 2=Second, 3=Third, 4=Fourth, 5=Last
        BREAK
      END
      LastDay = ThisDay
    END
  END

  !passed the date looking for the "Last"
  IF MONTH(ThisDate) <> st.wMonth OR ThisDate % 7 <> st.wDayOfWeek
    ThisDate = DATE(st.wMonth, LastDay, xYear)
  END
  RETURN ThisDate



!-----------------------------------------------------------------
! W2A
!
! Convert null terminated Wide (Unicode) string to ANSI string
!-----------------------------------------------------------------
SaTziClass.W2A     PROCEDURE(LONG lpWideString, *CSTRING lpANSIString)

  CODE
  !is the destination buffer too small?
?  ASSERT(WideCharToMultiByte(CP_ACP, WC_NONE,lpWideString,-1,lpANSIString,0,WC_NULL,WC_NULL) <= SIZE(lpANSISTring) )

  RETURN WideCharToMultiByte(CP_ACP,             |
                             WC_NONE,            |
                             lpWideString,       |
                             -1,                 | !Null Terminated input
                             lpANSIString,       |
                             SIZE(lpANSIString), |
                             WC_NULL,            |
                             WC_NULL             |
                            )

